@@ -16,6 +16,7 @@
 #include "ntp_malloc.h"
 #include "ntp_string.h"
 #include "ntp_syslog.h"
+#include "ntp_keyacc.h"
 
 #ifdef __GNUC__
 #define NTP_PRINTF(fmt, args) __attribute__((__format__(__printf__, fmt, args)))
@@ -39,6 +40,9 @@ extern	void	setup_logfile	(const char *);
 extern	void	errno_to_str(int, char *, size_t);
 #endif
 
+extern	int	xvsbprintf(char**, char* const, char const*, va_list) NTP_PRINTF(3, 0);
+extern	int	xsbprintf(char**, char* const, char const*, ...) NTP_PRINTF(3, 4);
+
 /*
  * When building without OpenSSL, use a few macros of theirs to
  * minimize source differences in NTP.
@@ -69,6 +73,7 @@ extern	int	authdecrypt	(keyid_t, u_int32 *, size_t, size_t);
 extern	size_t	authencrypt	(keyid_t, u_int32 *, size_t);
 extern	int	authhavekey	(keyid_t);
 extern	int	authistrusted	(keyid_t);
+extern	int	authistrustedip	(keyid_t, sockaddr_u *);
 extern	int	authreadkeys	(const char *);
 extern	void	authtrust	(keyid_t, u_long);
 extern	int	authusekey	(keyid_t, int, const u_char *);
@@ -95,29 +100,31 @@ extern	void	auth_prealloc_symkeys(int);
 extern	int	ymd2yd		(int, int, int);
 
 /* a_md5encrypt.c */
-extern	int	MD5authdecrypt	(int, const u_char *, u_int32 *, size_t, size_t);
-extern	size_t	MD5authencrypt	(int, const u_char *, u_int32 *, size_t);
-extern	void	MD5auth_setkey	(keyid_t, int, const u_char *, size_t);
+extern	int	MD5authdecrypt	(int, const u_char *, size_t, u_int32 *, size_t, size_t);
+extern	size_t	MD5authencrypt	(int, const u_char *, size_t, u_int32 *, size_t);
+extern	void	MD5auth_setkey	(keyid_t, int, const u_char *, size_t, KeyAccT *c);
 extern	u_int32	addr2refid	(sockaddr_u *);
 
 /* emalloc.c */
 #ifndef EREALLOC_CALLSITE	/* ntp_malloc.h defines */
 extern	void *	ereallocz	(void *, size_t, size_t, int);
-extern	void *	oreallocarray	(void *optr, size_t nmemb, size_t size);
+extern	void *	oreallocarrayxz	(void *optr, size_t nmemb, size_t size, size_t extra);
 #define	erealloczsite(p, n, o, z, f, l) ereallocz((p), (n), (o), (z))
 #define	emalloc(n)		ereallocz(NULL, (n), 0, FALSE)
 #define	emalloc_zero(c)		ereallocz(NULL, (c), 0, TRUE)
 #define	erealloc(p, c)		ereallocz((p), (c), 0, FALSE)
 #define erealloc_zero(p, n, o)	ereallocz((p), (n), (o), TRUE)
-#define ereallocarray(p, n, s)	oreallocarray((p), (n), (s))
-#define eallocarray(n, s)	oreallocarray(NULL, (n), (s))
+#define ereallocarray(p, n, s)	oreallocarrayxz((p), (n), (s), 0)
+#define eallocarray(n, s)	oreallocarrayxz(NULL, (n), (s), 0)
+#define ereallocarrayxz(p, n, s, x)	oreallocarrayxz((p), (n), (s), (x))
+#define eallocarrayxz(n, s, x)	oreallocarrayxz(NULL, (n), (s), (x))
 extern	char *	estrdup_impl(const char *);
 #define	estrdup(s)		estrdup_impl(s)
 #else
 extern	void *	ereallocz	(void *, size_t, size_t, int,
 				 const char *, int);
-extern	void *	oreallocarray	(void *optr, size_t nmemb, size_t size,
-				 const char *, int);
+extern	void *	oreallocarrayxz	(void *optr, size_t nmemb, size_t size,
+				 size_t extra, const char *, int);
 #define erealloczsite		ereallocz
 #define	emalloc(c)		ereallocz(NULL, (c), 0, FALSE, \
 					  __FILE__, __LINE__)
@@ -127,9 +134,13 @@ extern	void *	oreallocarray	(void *optr, size_t nmemb, size_t size,
 					  __FILE__, __LINE__)
 #define	erealloc_zero(p, n, o)	ereallocz((p), (n), (o), TRUE, \
 					  __FILE__, __LINE__)
-#define ereallocarray(p, n, s)	oreallocarray((p), (n), (s), \
+#define ereallocarray(p, n, s)	oreallocarrayxz((p), (n), (s), 0, \
 					  __FILE__, __LINE__)
-#define eallocarray(n, s)	oreallocarray(NULL, (n), (s), \
+#define eallocarray(n, s)	oreallocarrayxz(NULL, (n), (s), 0, \
+					  __FILE__, __LINE__)
+#define ereallocarrayxz(p, n, s, x)	oreallocarrayxz((p), (n), (s), (x), \
+					  __FILE__, __LINE__)
+#define eallocarrayxz(n, s, x)	oreallocarrayxz(NULL, (n), (s), (x), \
 					  __FILE__, __LINE__)
 extern	char *	estrdup_impl(const char *, const char *, int);
 #define	estrdup(s) estrdup_impl((s), __FILE__, __LINE__)
@@ -141,6 +152,7 @@ extern	int	atouint		(const char *, u_long *);
 extern	int	hextoint	(const char *, u_long *);
 extern	const char *	humanlogtime	(void);
 extern	const char *	humantime	(time_t);
+extern int	is_ip_address	(const char *, u_short, sockaddr_u *);
 extern	char *	mfptoa		(u_int32, u_int32, short);
 extern	char *	mfptoms		(u_int32, u_int32, short);
 extern	const char * modetoa	(size_t);
@@ -197,7 +209,7 @@ extern int	authnumfreekeys;
 extern keyid_t	cache_keyid;		/* key identifier */
 extern int	cache_type;		/* key type */
 extern u_char *	cache_secret;		/* secret */
-extern u_short	cache_secretsize;	/* secret octets */
+extern size_t	cache_secretsize;	/* secret octets */
 extern u_short	cache_flags;		/* KEY_ bit flags */
 
 /* getopt.c */
